@@ -1,6 +1,6 @@
-/* Time-stamp: <2008-08-30 21:17:51 poser>
+/* Time-stamp: <2009-04-19 13:01:00 poser>
  *
- * Copyright (C) 2007 William J. Poser (billposer@alum.mit.edu)
+ * Copyright (C) 2008 William J. Poser (billposer@alum.mit.edu)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 3 of the GNU General Public License as published by
@@ -67,7 +67,7 @@ void SetFormat(char *fs,
 {
   regex_t rx;
   
-  /* First, check for old  flag characters */
+  /* First, check for old flag characters */
 
   if(strlen(fs) == 1) {
     switch(fs[0]) {
@@ -156,6 +156,10 @@ void SetFormat(char *fs,
       *FType = CHENT;
       *AllHTMLP = 1;
       return;
+    case '0':
+      *FType = UTF8ANGLE;
+      *UTF8Type = 5;
+      return;
     case '1':
       *FType = CLSX;
       return;
@@ -181,6 +185,9 @@ void SetFormat(char *fs,
     case '8':
       *FType = OOXML;
       return;
+    case '9':
+      *FType = PCTUX;
+      return;
     default:
       fprintf(stderr,"Unknown format %s\n",fs);
       exit(BADOPTIONARG);
@@ -263,6 +270,10 @@ void SetFormat(char *fs,
   if(!regexec(&rx,fs,0,NULL,0)) {*FType = JuUX;return;}
   regfree(&rx);
 
+  regcomp(&rx,"^%u[[:xdigit:]]{4,}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED); /* PCTUX */
+  if(!regexec(&rx,fs,0,NULL,0)) {*FType = PCTUX;return;}
+  regfree(&rx);
+
   regcomp(&rx,"^U\\+[[:xdigit:]]{4,}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED); /* UPLX */
   if(!regexec(&rx,fs,0,NULL,0)) {*FType = UPLX;return;}
   regfree(&rx);
@@ -307,6 +318,10 @@ void SetFormat(char *fs,
   if(!regexec(&rx,fs,0,NULL,0)) {*FType = BYTEH;return;}
   regfree(&rx);
 
+  regcomp(&rx,"^(<[[:xdigit:]]{2}>){1,3}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* UTF8ANGLE */
+  if(!regexec(&rx,fs,0,NULL,0)) {*FType = UTF8ANGLE;return;}
+  regfree(&rx);
+
   regcomp(&rx,"^&[^0-9]+;$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* Non-numeric Character entity */
   if(!regexec(&rx,fs,0,NULL,0)) {*UseEntitiesP = 1;return;}
   regfree(&rx);
@@ -438,6 +453,7 @@ void ListFormatArguments (short ToAsciiP) {
   fprintf(stderr, _(" prefix \\U outside BMP, \\u within, hexadecimal (U+0000-U+FFFF)\tL\n"));
   fprintf(stderr, _(" prefix U hexadecimal (U00E9)\tE\n"));
   fprintf(stderr, _(" prefix u hexadecimal (u00E9)\tF\n"));
+  fprintf(stderr, _(" prefix %%u hexadecimal (%%u00E9)\t9\n"));
   fprintf(stderr, _(" prefix U+ hexadecimal (U+00E9)\tP\n"));
   fprintf(stderr, _(" prefix X with hexadecimal in single quotes (X\'00E9\')\tG\n"));
   fprintf(stderr, _(" prefix 16# and suffix # hexadecimal (16#00E9#)\t6\n"));
@@ -456,6 +472,7 @@ void ListFormatArguments (short ToAsciiP) {
   fprintf(stderr, _(" hexadecimal UTF-8 with each byte's hex preceded by an =-sign (=C3=A9).\n\t\tRFC 2045 Quoted Printable.\tI\n"));
   fprintf(stderr, _(" hexadecimal UTF-8 with each byte's hex preceded by a %%-sign  (%%C3%%A9)\n\t\tRFC 2396  URI escape format.\tJ\n"));
   fprintf(stderr, _(" hexadecimal UTF-8 with each byte's hex preceded by a backslash-x  (\\xC3\\xA9)\n\t\tApache log format.\t7\n"));
+  fprintf(stderr, _(" hexadecimal UTF-8 with each byte's hex surrounded by angle brackets  (<C3><A9>)\n\t\t\t0\n"));
   fprintf(stderr, _(" octal UTF-8 with backslash escapes (\303\251)\tK\n"));
 
   if(ToAsciiP) fprintf(stderr, _(" HTML character entities where possible (&eacute;), else numeric\tQ\n"));
