@@ -1,8 +1,8 @@
-/* Time-stamp: <2008-08-30 21:21:02 poser>
+/* Time-stamp: <2009-03-25 18:44:10 poser>
  *
- * Convert text containing various 7-bit ASCII escapes to UTF-7 Unicode.
+ * Convert text containing various 7-bit ASCII escapes to UTF-8 Unicode.
  *
- * Copyright (C) 2005-2008 William J. Poser (billposer@alum.mit.edu)
+ * Copyright (C) 2005-2009 William J. Poser (billposer@alum.mit.edu)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 3 of the GNU General Public License
@@ -86,7 +86,7 @@ ShowVersion(FILE *fp)
 }
 
 void Copyright() {
-  fprintf(stderr,"Copyright (C) 2004-2008 William J. Poser\n");
+  fprintf(stderr,"Copyright (C) 2004-2009 William J. Poser\n");
   fprintf(stderr,"This program is free software; you can redistribute\n\
 it and/or modify it under the terms of version 3 of\n\
 the GNU General Public License as published by the\n\
@@ -99,7 +99,7 @@ void
 ShowUsage(void){
   fprintf(stderr,_("This program is a filter which converts 7-bit ASCII text\n\
 containing various representations for non-ASCII characters\nto UTF-8 Unicode.\n"));
-  fprintf(stderr,_("Usage: %s [flags]\n"),pgname);
+  fprintf(stderr,_("Usage: %s [flags] (<input filename>)\n"),pgname);
   fprintf(stderr,_("       -a <format specification>.\n"));
   fprintf(stderr,_("       -h Print this usage message.\n"));
   fprintf(stderr,_("       -L List format codes.\n"));
@@ -188,23 +188,27 @@ static char *Formats [] = {
 "\\d%03d\\d%03d\\d%03d",	/* BYTED */
 "\\x%02x\\x%02x\\x%02x",	/* BYTEX */
 "_x%4lX_",	/* OOXML */
+"%%u%lX",		/* PCTUX */
 "&%[abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789];", /* CHENT */
 "=%2lX", 		/* UTF-8  - Ifmt */
 "%%%2lX", 		/* UTF-8  - Jfmt */
 "\\%3lo", 		/* UTF-8  - Kfmt */
-"\\x%2lX" 		/* UTF-8  - APACHE */
+"\\x%2lX", 		/* UTF-8  - APACHE */
+"<%2lX>" 		/* UTF-8  - UTF8ANGLE */
 };
 
+#define AFMTSIZE (67+2+1+2)
 
 int main (int ac, char *av[])
 {
   char *SplitFormat = "\\%1[uU]%X%n"; /* This is for BMPSplit */
 
   char *fmt = Formats[STDX];		/* Default is plain hex format */
-  char afmt [67+2+1+2];
+  char afmt [AFMTSIZE];
   char aHfmt [8+2+1];
   char aDfmt [8+2+1];
   char cbuf[5];
+  FILE *infp;
 
   UTF32 num;
   int oc;			/* Command line option flag */
@@ -251,6 +255,10 @@ int main (int ac, char *av[])
   extern void ListFormatArguments(short);
   extern void SetFormat(char *, int *, short *,int *, short *, short *);
 
+#ifdef DEBUGBUILD
+  fprintf(stderr,"Execution has begun.\n");fflush(stderr);
+#endif
+
   opterr = 0;
 
 #ifdef HAVE_SETLOCALE
@@ -272,8 +280,10 @@ int main (int ac, char *av[])
 	 exit(BADOPTIONARG);
        }
        fmt = Formats[FType];
-       if((FType == IFMT) || (FType == JFMT) || (FType == KFMT) || (FType == APACHE)) UTF8ValueP =1;
-       if(FType == JFMT) {cbuf[0] = '0'; cbuf[1] = 'x';}
+       if( (FType == IFMT) || (FType == JFMT) ||
+	   (FType == KFMT) || (FType == APACHE) ||
+	   (FType == UTF8ANGLE)) UTF8ValueP =1;
+       if( (FType == JFMT) || (FType == UTF8ANGLE)) {cbuf[0] = '0'; cbuf[1] = 'x';}
        if(FType == KFMT) {cbuf[0] = '\\';}
        if(FType == APACHE) {cbuf[0] = '0';}
        break;
@@ -317,6 +327,19 @@ int main (int ac, char *av[])
      }
    }
 
+   if(optind < ac) {
+     infp = fopen(av[optind],"r");
+     if (infp == NULL) {
+       fprintf(stderr,"unable to open input file %s\n",av[optind]);
+       exit(OPENERROR);
+     }
+   }
+   else infp = stdin;
+
+#ifdef DEBUGBUILD
+   fprintf(stderr,"Command-line arguments processed.\n");fflush(stderr);
+#endif
+
    if( (FType == RAWX) && (!PureP) ) {
      fprintf(stderr,_("It isn't possible reliably to parse raw hex unicode out of ASCII text.\n"));
      exit(BADOPTION);
@@ -327,22 +350,33 @@ int main (int ac, char *av[])
      exit(BADOPTION);
    }
 
+#ifdef DEBUGBUILD
+   fprintf(stderr,"Sanity checks completed.\n");fflush(stderr);
+#endif
+
    if(AllHTMLP) {
      sprintf(aDfmt,"%s%%n",Formats[HTMLD]);
      sprintf(aHfmt,"%s%%n",Formats[HTMLX]);
    }
 
-   sprintf(afmt,"%s%%n",fmt);	/* Add %n for NConsumed */
+#ifdef DEBUGBUILD
+   fprintf(stderr,"fmt = %s\n",fmt);fflush(stderr); /* debug */
+#endif
+
+   snprintf(afmt,AFMTSIZE,"%s%%n",fmt);	/* Add %n for NConsumed */
 
    /*
     * This is the case in which the input consists entirely of escapes
     * except for arbitrary (but non-null) amounts of intervening whitespace.
     */
 
+#ifdef DEBUGBUILD
+   fprintf(stderr,"Setup completed.\n");fflush(stderr); /* debug */
+#endif
    if(PureP) {
      GetWordLineNo = 1;
      while(1){
-       str = Get_Word(stdin,&Word_Length,&eof);
+       str = Get_Word(infp,&Word_Length,&eof);
        if(eof) break; 
        if(Word_Length == 0) continue;
        TokenNumber++;
@@ -363,7 +397,7 @@ int main (int ac, char *av[])
 	 }
 	 else Converted = 1;
        }
-       else if( (BYTEO == FType) || (BYTED == FType) || (BYTEH == FType)) {
+       else if( (BYTEO == FType) || (BYTED == FType) || (BYTEH == FType) || (UTF8ANGLE == FType)) {
 	 Converted = sscanf(str,afmt,&b1,&b2,&b3,&NConsumed);
 	 switch(Converted)
 	   {
@@ -429,7 +463,7 @@ int main (int ac, char *av[])
      fprintf(stderr,"Failed to allocate buffer for input line.\n");
      exit(2);
    }
-   while ((read = getline(&lbuf, &len, stdin)) != -1) {
+   while ((read = getline(&lbuf, &len, infp)) != -1) {
      AddNewlineP = 1;
      LineNo++;
      last = read - 1;
@@ -563,7 +597,7 @@ int main (int ac, char *av[])
 	 }
 	 else putchar(*iptr++);
        } /* End of Qfmt case */
-       else if( (BYTEO == FType) || (BYTED == FType) || (BYTEH == FType)) {
+       else if( (BYTEO == FType) || (BYTED == FType) || (BYTEH == FType) ) {
 	 Converted=sscanf(iptr,afmt,&b1,&b2,&b3,&NConsumed);
 	 fprintf(stderr,"Converted = %d\n",Converted);fflush(stderr);
 	 switch(Converted)
