@@ -1,9 +1,9 @@
-/* Time-stamp: <2008-08-30 20:27:37 poser>
+/* Time-stamp: <2009-04-22 00:41:23 poser>
  *
  * Converts UTF-8 Unicode to pure 7-bit ASCII using any of a number
  * of different representations. 
  * 
- * Copyright (C) 2004-2008 William J. Poser (billposer@alum.mit.edu)
+ * Copyright (C) 2004-2009 William J. Poser (billposer@alum.mit.edu)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 3 of the GNU General Public License as published by
@@ -26,6 +26,8 @@
 #include <unistd.h>
 #include <errno.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <string.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -92,7 +94,7 @@ ShowVersion(FILE *fp)
 
 
 void Copyright (void) {
-  fprintf(stderr,"Copyright (C) 2004-2008 William J. Poser\n");
+  fprintf(stderr,"Copyright (C) 2004-2009 William J. Poser\n");
   fprintf(stderr,"This program is free software; you can redistribute\n\
 it and/or modify it under the terms of version 3 of\n\
 the GNU General Public License as published by the\n\
@@ -109,7 +111,7 @@ format except for newline and the space characters (space, tab\n\
 ideographic space, ethiopic word space and ogham space mark).\n\
 Options allow conversion of the ASCII characters as well\n\
 and for conversion even of newline and space characters.\n"));
-  fprintf(stderr,_("Usage: %s [flags]\n"),pgname);
+  fprintf(stderr,_("Usage: %s [flags] (<input file name>)\n"),pgname);
   fprintf(stderr,_("       -h Print this usage message.\n"));
   fprintf(stderr,_("       -v Print version information.\n"));
   fprintf(stderr,_("       -L List format specifications.\n"));
@@ -283,7 +285,9 @@ static char *Formats [] = {
   "\\x%02hhx\\x%02hhx\\x%02hhx",/* BYTEH */
   "\\x%02hhX\\x%02hhX\\x%02hhX",
   "_x%04x_", /* OOXML */
-  "_x%04X_" 
+  "_x%04X_",
+  "%%u%04x", 	/* PCTUX */
+  "%%u%04X"
   };
 
 /*  Returns 1 if it handled the character, 0 if not. */
@@ -940,8 +944,7 @@ int AscifyDiacritics(UTF32 c) {
     case 0x1EB7:
       putchar(0x61);
       break;
-    case 0x0062:		/* b */
-    case 0x0180:
+    case 0x0180:		/* b */
     case 0x0183:
     case 0x0253:
     case 0x1E03:
@@ -1765,7 +1768,6 @@ int AscifyStyle(UTF32 c) {
     case 0x1D67D:
       putchar(0x4E);		/* N */
       break;
-    case 0x004F:
     case 0x2205:
     case 0xFF2F:
     case 0x1D40E:
@@ -1865,7 +1867,6 @@ int AscifyStyle(UTF32 c) {
     case 0x1D683:
       putchar(0x54);		/* T */
       break;
-    case 0x0055:
     case 0xFF35:
     case 0x1D414:
     case 0x1D448:
@@ -1930,7 +1931,6 @@ int AscifyStyle(UTF32 c) {
     case 0x1D687:
       putchar(0x58);		/* X */
       break;
-    case 0x0059:
     case 0x2144:
     case 0xFF39:
     case 0x1D418:
@@ -1978,8 +1978,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D68B:
       putchar(0x62);
       break;
-    case 0x0063:		/* c */
-    case 0x1D04:
+    case 0x1D04:		/* c */
     case 0xFF43:
     case 0x1D41C:
     case 0x1D450:
@@ -1996,8 +1995,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D68C:
       putchar(0x63);
       break;
-    case 0x0064:		/* d */
-    case 0x1D05:
+    case 0x1D05:		/* d */
     case 0x2146:
     case 0xFF44:
     case 0x1D41D:
@@ -2015,8 +2013,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D68D:
       putchar(0x64);
       break;
-    case 0x0065:		/* e */
-    case 0x1D07:
+    case 0x1D07:		/* e */
     case 0x212F:
     case 0x2147:
     case 0xFF45:
@@ -2033,8 +2030,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D68E:
       putchar(0x65);
       break;
-    case 0x0066:		/* f */
-    case 0xFF46:
+    case 0xFF46:		/* f */
     case 0x1D41F:
     case 0x1D453:
     case 0x1D487:
@@ -2050,8 +2046,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D68F:
       putchar(0x66);
       break;
-    case 0x0067:		/* g */
-    case 0xFF47:
+    case 0xFF47:		/* g */
     case 0x210A:
     case 0x1D420:
     case 0x1D454:
@@ -2067,8 +2062,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D690:
       putchar(0x67);
       break;
-    case 0x0068:		/* h */
-    case 0xFF48:
+    case 0xFF48:		/* h */
     case 0x1D421:
     case 0x1D489:
     case 0x1D4BD:
@@ -2120,8 +2114,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D693:
       putchar(0x6A);		/* j */
       break;
-    case 0x006B:		/* k */
-    case 0x1D0B:
+    case 0x1D0B:		/* k */
     case 0xFF4B:
     case 0x1D424:
     case 0x1D458:
@@ -2138,8 +2131,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D694:
       putchar(0x6B);
       break;
-    case 0x006C:		/* l */
-    case 0x2113:
+    case 0x2113:		/* l */
     case 0xFF4C:
     case 0x1D425:
     case 0x1D459:
@@ -2156,8 +2148,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D695:
       putchar(0x6C);
       break;
-    case 0x006D:		/* m */
-    case 0x1D0D:
+    case 0x1D0D:		/* m */
     case 0xFF4D:
     case 0x1D426:
     case 0x1D45A:
@@ -2190,8 +2181,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D697:
       putchar(0x6E);		/* n */
       break;
-    case 0x006F:		/* o */
-    case 0x1D0F:
+    case 0x1D0F:		/* o */
     case 0x2134:
     case 0xFF4F:
     case 0x1D428:
@@ -2208,8 +2198,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D698:
       putchar(0x6F);
       break;
-    case 0x0070:		/* p */
-    case 0x1D18:
+    case 0x1D18:		/* p */
     case 0xFF50:
     case 0x213C:
     case 0x1D429:
@@ -2227,8 +2216,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D699:
       putchar(0x70);
       break;
-    case 0x0071:		/* q */
-    case 0xFF51:
+    case 0xFF51:		/* q */
     case 0x1D42A:
     case 0x1D45E:
     case 0x1D492:
@@ -2244,8 +2232,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D69A:
       putchar(0x71);
       break;
-    case 0x0072:		/* r */
-    case 0xFF52:
+    case 0xFF52:		/* r */
     case 0x1D42B:
     case 0x1D45F:
     case 0x1D493:
@@ -2261,8 +2248,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D69B:
       putchar(0x72);
       break;
-    case 0x0073:		/* s */
-    case 0xFF53:
+    case 0xFF53:		/* s */
     case 0x1D42C:
     case 0x1D460:
     case 0x1D494:
@@ -2278,8 +2264,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D69C:
       putchar(0x73);
       break;
-    case 0x0074:		/* t */
-    case 0x1D1B:
+    case 0x1D1B:		/* t */
     case 0xFF54:
     case 0x1D42D:
     case 0x1D461:
@@ -2296,8 +2281,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D69D:
       putchar(0x74);
       break;
-    case 0x0075:		/* u */
-    case 0x1D1C:
+    case 0x1D1C:		/* u */
     case 0xFF55:
     case 0x1D42E:
     case 0x1D462:
@@ -2314,8 +2298,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D69E:
       putchar(0x75);
       break;
-    case 0x0076:		/* v */
-    case 0x1D20:
+    case 0x1D20:		/* v */
     case 0xFF56:
     case 0x1D42F:
     case 0x1D463:
@@ -2332,8 +2315,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D69F:
       putchar(0x76);
       break;
-    case 0x0077:		/* w */
-    case 0x1D21:
+    case 0x1D21:		/* w */
     case 0x24B2:
     case 0x24E6:
     case 0xFF57:
@@ -2352,8 +2334,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D6A0:
       putchar(0x77);
       break;
-    case 0x0078:		/* x */
-    case 0xFF58:
+    case 0xFF58:		/* x */
     case 0x1D431:
     case 0x1D465:
     case 0x1D499:
@@ -2385,8 +2366,7 @@ int AscifyStyle(UTF32 c) {
     case 0x1D6A2:
       putchar(0x79);		/* y */
       break;
-    case 0x007A:		/* z */
-    case 0x1D22:
+    case 0x1D22:		/* z */
     case 0xFF5A:
     case 0x1D433:
     case 0x1D467:
@@ -2755,7 +2735,6 @@ int main (int ac, char *av[])
   char *fmt = NULL;
   short dummy;			/* For compatibility with ascii2uni */
 
-
   unsigned char b1,b2,b3; /* The low three bytes of a 4 byte UTF-32 character */
 
   unsigned long ByteCnt;
@@ -2907,7 +2886,16 @@ int main (int ac, char *av[])
       exit(BADOPTION);
     }
   } 
-  infd = fileno(stdin);
+
+  if(optind < ac) {
+    infd = open(av[optind],O_RDONLY);
+    if (infd < 0) {
+      fprintf(stderr,"unable to open input file %s\n",av[optind]);
+      exit(OPENERROR);
+    }
+  }
+  else infd = fileno(stdin);
+
 
   if(VerboseP) {
     if(SubCnt) fprintf(stderr,"Custom substitutions defined: %d\n",SubCnt);
@@ -3019,6 +3007,9 @@ int main (int ac, char *av[])
 	    case 4:
 	      printf("\\x%02X",ch);
 	      break;
+	    case 5:
+	      printf("<%02X>",ch);
+	      break;
 	    }
 	  }
 	  if(AddWhitespaceP) putchar(' ');
@@ -3044,6 +3035,9 @@ int main (int ac, char *av[])
 	  case 4:
 	    printf("\\x%02X",ch);
 	    break;
+	  case 5:
+	    printf("<%02X>",ch);
+	    break;
 	  }
 	}
 	if(AddWhitespaceP) putchar(' ');
@@ -3084,12 +3078,12 @@ int main (int ac, char *av[])
 	continue;
       }
     }
-	if(MakeP) {
-	  if(MakeAscii(c)) {
-	    SubstitutedCnt++;
-	    continue;
-	  }
-	}
+    if(MakeP) {
+      if(MakeAscii(c)) {
+	SubstitutedCnt++;
+	continue;
+      }
+    }
     if(AscifyEquivP) {
       if(AscifyEquiv(c)) {
 	SubstitutedCnt++;
